@@ -27,13 +27,12 @@
 
 #define event_ends(event) ((event)->start + (event)->duration)
 
-
-GSList *found_node;
+static GSList *found_node;
 
 /**
  * Make new set used by overlapping set 
  */
-GNode *
+static GNode *
 make_set (event_t e)
 {
   GSList *set = NULL;
@@ -43,7 +42,7 @@ make_set (event_t e)
 
 }
 
-gboolean
+static gboolean
 events_overlap (event_t ev1, event_t ev2)
 {				
   /* Event overlaps if and only if they intersect */
@@ -70,8 +69,7 @@ events_overlap (event_t ev1, event_t ev2)
     }
 }
 
-
-gboolean
+static gboolean
 find_set_in_list (GNode * node, gpointer data)
 {
   GSList *iter, *ol_set;
@@ -104,7 +102,7 @@ find_set_in_list (GNode * node, gpointer data)
 }
 
 /* Returns the overlapping set */
-GSList *
+static GSList *
 find_set (GNode * root, event_t ev)
 {
   found_node = NULL;
@@ -112,18 +110,16 @@ find_set (GNode * root, event_t ev)
   g_node_traverse (root, G_IN_ORDER, G_TRAVERSE_ALL, -1, find_set_in_list, ev);
 
   return found_node;
-
-
 }
 
 
-GSList *
+static GSList *
 union_set (GSList * set, const event_t ev)
 {
   return (g_slist_append (set, ev));
 }
 
-GNode *
+static GNode *
 find_overlapping_sets (GSList * events)
 {
   GNode *root;
@@ -152,14 +148,14 @@ find_overlapping_sets (GSList * events)
   return root;
 }
 
-void
+static void
 show_event (const struct day_render *dr, const ev_rec_t event_rectangle,
 	    GdkGC * gc)
 {
   guint length;
 
   event_details_t evd;
-  gchar buf[250], *buffer;
+  gchar *buffer;
   PangoLayout *pl;
   GdkRectangle gr;
   gint arc_size;
@@ -168,12 +164,8 @@ show_event (const struct day_render *dr, const ev_rec_t event_rectangle,
 
   pl = gtk_widget_create_pango_layout (dr->widget, NULL);
 
-
   evd = event_db_get_details (event_rectangle->event);
-  snprintf (buf, sizeof (buf), "<span size='small'>%s</span>", evd->summary);
-  buffer = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-  if (!buffer)
-      buffer = g_strdup(buf);
+  buffer = g_strdup_printf ("<span size='small'>%s</span>", evd->summary);
 
   length = event_rectangle->height - 1;
   /* Rectangle used to write appointment summary */
@@ -264,6 +256,7 @@ show_event (const struct day_render *dr, const ev_rec_t event_rectangle,
                    GDK_RGB_DITHER_NORMAL, 0, 0);
         }
     }
+  g_object_unref (pl);
   g_free (buffer);
 }
 
@@ -283,22 +276,18 @@ draw_appointments (struct day_render *dr)
 }
 
 
-
 /*
  * Caption functions.
  */
 
 /* Caption Constructor*/
-#define caption_new(day, width, height, offset,gc,widget,bell) caption_with_pango_new((day),(width),(height),(offset),(gc),(widget),bell ,NULL)
-
 /** 
- * Caption constructor specifing pango layout
+ * Caption constructor
  * widget is the widget where you want pango layout will be drawn
  */
 caption_t
-caption_with_pango_new (guint day, guint width, guint height, GdkPoint offset,
-			GdkGC * gc, GtkWidget * widget, GdkPixbuf * bell,
-			PangoLayout * pl)
+caption_new (guint day, guint width, guint height, GdkPoint offset,
+	     GdkGC * gc, GtkWidget * widget, GdkPixbuf * bell)
 {
   caption_t this;
 
@@ -307,20 +296,13 @@ caption_with_pango_new (guint day, guint width, guint height, GdkPoint offset,
   if (this == NULL)
     return NULL;
 
-  if (pl != NULL)
+  if (widget == NULL)
     {
-      this->pl = pl;
+      g_free (this);
+      return NULL;
     }
-  else
-    {
-      if (widget == NULL)
-	{
-	  g_free (this);
-	  return NULL;
-	}
 
-      this->pl = gtk_widget_create_pango_layout (widget, NULL);
-    }
+  this->pl = gtk_widget_create_pango_layout (widget, NULL);
   this->draw = widget;
   this->day = day;
   this->height = height;
@@ -331,6 +313,14 @@ caption_with_pango_new (guint day, guint width, guint height, GdkPoint offset,
   return this;
 }
 
+void
+caption_delete (caption_t capt)
+{
+  g_object_unref (capt->pl);
+  g_free (capt);
+}
+
+#if 0
 void
 caption_set_pango (caption_t this, PangoLayout * pl)
 {
@@ -340,18 +330,13 @@ caption_set_pango (caption_t this, PangoLayout * pl)
 void
 caption_show (caption_t this)
 {
-  char buf[40], *buffer;
+  char *buffer;
   PangoRectangle pr;
   GdkRectangle gr;
 
-  buffer = (char *) g_malloc (sizeof (char) * 256);
-
   /* Displays day number */
-  snprintf (buf, sizeof (buf), "<span size='%d'>%d</span>",
-	    this->height * 800, this->day);
-
-
-  buffer = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+  buffer = g_strdup_printf ("<span size='%d'>%d</span>",
+			    this->height * 800, this->day);
 
   pango_layout_set_markup (this->pl, buffer, strlen (buffer));
   pango_layout_get_pixel_extents (this->pl, &pr, NULL);
@@ -371,12 +356,13 @@ caption_show (caption_t this)
 
   if (this->day == 19)
     gdk_draw_rectangle (this->draw->window, this->gc, TRUE,
-			this->offset.x + gr.width * 1.2,
+\			this->offset.x + gr.width * 1.2,
 			this->offset.y + this->height * .5,
 			this->height * 0.8, this->height * 0.8);
 
   g_free (buffer);
 }
+#endif
 
 /* 
  * Returns true if event started today.
@@ -408,9 +394,7 @@ event_starts_today (const struct day_render *dr, const event_t ev)
     }
 }
 
-
-
-ev_rec_t
+static ev_rec_t
 event_rect_new (const struct day_render * dr, const event_t event,
 		gint column, gint columns)
 {
@@ -472,10 +456,7 @@ event_rect_delete (ev_rec_t ev)
   g_free (ev);
 }
 
-
-
-
-GSList *
+static GSList *
 ol_sets_to_rectangles (const struct day_render *dr, GNode * node)
 {
   GSList *iter, *ev_rects = NULL;
@@ -601,6 +582,9 @@ day_render_new (GtkWidget * widget,
   dr->offset.x = page->time_width;
   dr->offset.y = 0;
 
+  g_object_ref (dr->normal_gc);
+  g_object_ref (dr->ol_gc);
+
   pbuf = gpe_find_icon ("bell");
 
 
@@ -633,9 +617,11 @@ day_render_delete (struct day_render *dr)
          iter = iter->next;
        }
 
+      g_object_unref (dr->ol_gc);
+      g_object_unref (dr->normal_gc);
       g_slist_free (dr->event_rectangles);
       g_slist_free (dr->events);
-      g_free (dr->capt);
+      caption_delete (dr->capt);
       g_free (dr);
     }
 }
