@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001, 2002 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2001, 2002, 2006 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -11,7 +11,6 @@
 #include <time.h>
 #include <gpe/event-db.h>
 
-extern GList *times;
 extern time_t viewtime;
 extern gboolean force_today;
 
@@ -35,6 +34,7 @@ extern GtkWidget *main_window, *pop_window;
 extern GdkFont *yearfont, *datefont;
 extern gchar *strftime_strdup_utf8_locale (const char *fmt, struct tm *tm);
 extern gchar *strftime_strdup_utf8_utf8 (const char *fmt, struct tm *tm);
+extern guint day_of_week(guint year, guint month, guint day);
 
 #define SECONDS_IN_DAY (24*60*60)
 
