@@ -57,7 +57,6 @@
 
 extern gboolean gpe_calendar_start_xsettings (void);
 
-GList *times;
 time_t viewtime;
 gboolean force_today = FALSE;
 gboolean just_new = FALSE;
@@ -91,6 +90,22 @@ gint ccpos = 0;
 static guint nr_days[] = { 31, 28, 31, 30, 31, 30, 
 			   31, 31, 30, 31, 30, 31 };
 
+guint
+day_of_week (guint year, guint month, guint day)
+{
+  guint result;
+
+  if (month < 3) 
+    {
+      month += 12;
+      --year;
+    }
+
+  result = day + (13 * month - 27)/5 + year + year/4
+    - year/100 + year/400;
+  return ((result + 6) % 7);
+}
+
 guint
 days_in_month (guint year, guint month)
 {
@@ -257,7 +272,7 @@ new_appointment (void)
 }
 
 static void
-set_today(void)
+set_today (void)
 {
   static time_t selected_time;
 
@@ -270,16 +285,18 @@ set_today(void)
   else
     viewtime = selected_time;
   
-  set_time_all_views();
+  set_time_all_views ();
+  day_view_scroll (TRUE);
 }
 
 void
-set_time_and_day_view(time_t selected_time)
+set_time_and_day_view (time_t selected_time)
 {
-  viewtime=selected_time;
+  viewtime = selected_time;
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (day_button), TRUE);
   new_view (day);
-  update_current_view();
+  update_current_view ();
+  day_view_scroll (TRUE);
 }
 
 static void
@@ -539,7 +556,7 @@ main (int argc, char *argv[])
   osso_context_t *osso_context;
 #endif
 
-  guint hour, skip=0, uid=0;
+  guint skip=0, uid=0;
   int option_letter;
   gboolean schedule_only=FALSE;
   extern char *optarg;
@@ -599,20 +616,6 @@ main (int argc, char *argv[])
       exit (EXIT_SUCCESS);
     }
 	
-  for (hour = 0; hour < 24; hour++)
-    {
-      struct tm tm;
-      time_t t=time(NULL);
-
-      localtime_r (&t, &tm);
-      tm.tm_hour = hour;
-      tm.tm_min = 0;
-      times = g_list_append (times, strftime_strdup_utf8_locale (TIMEFMT, &tm));
-      tm.tm_hour = hour;
-      tm.tm_min = 30;
-      times = g_list_append (times, strftime_strdup_utf8_locale (TIMEFMT, &tm));
-    }
-
   vcal_export_init();
     
   vbox = gtk_vbox_new (FALSE, 0);
@@ -824,6 +827,7 @@ main (int argc, char *argv[])
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (day_button), TRUE);
   gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON (today_button), FALSE);
   new_view (day);
+  day_view_scroll (TRUE);
 
   gtk_main ();
 
