@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001, 2002, 2004, 2005 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2001, 2002, 2004, 2005, 2006 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -60,9 +60,7 @@ struct week_day
 
 
 static gint
-draw_expose_event (GtkWidget *widget,
-		   GdkEventExpose *event,
-		   gpointer user_data)
+draw_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 {
   GtkDrawingArea *darea;
   GdkDrawable *drawable;
@@ -133,7 +131,7 @@ draw_expose_event (GtkWidget *widget,
     
           pango_layout_set_width (pl, max_width * PANGO_SCALE);
           pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
-          pango_layout_get_pixel_extents (pl, &pr, NULL);
+          pango_layout_get_pixel_extents (pl, NULL, &pr);
           if (pr.width > day_label_width)
             day_label_width = pr.width;
         }
@@ -158,7 +156,7 @@ draw_expose_event (GtkWidget *widget,
 
       pango_layout_set_width (pl, max_width * PANGO_SCALE);
       pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
-      pango_layout_get_pixel_extents (pl, &pr, NULL);
+      pango_layout_get_pixel_extents (pl, NULL, &pr);
       x = 4;
 //      x = wide_mode ? 4 : max_width - pr.width - 8;
       gtk_paint_layout (widget->style,
@@ -179,7 +177,6 @@ draw_expose_event (GtkWidget *widget,
 	  for (iter = week_days[day].events; iter; iter = iter->next)
 	    {
 	      guint height = 0;
-	      char buf[256];
 	      struct tm tm;
 	      event_t ev = iter->data;
 	      event_details_t evd = event_db_get_details (ev);
@@ -190,11 +187,13 @@ draw_expose_event (GtkWidget *widget,
 		  if (ev->mark == FALSE)
 		    {
 		      gchar *buffer;
+
 		      localtime_r (&ev->start, &tm);
-		      strftime (buf, sizeof (buf), "%H:%M", &tm);
-		      buffer = g_locale_to_utf8 (buf, -1, NULL,
-						 NULL, NULL);
-		      pango_layout_set_text (pl_evt, buffer, strlen (buffer));
+
+		      buffer = strftime_strdup_utf8_locale ("%H:%M", &tm);
+		      pango_layout_set_text (pl_evt, buffer, -1);
+		      g_free (buffer);
+
 		      gtk_paint_layout (widget->style,
 					widget->window,
 					GTK_WIDGET_STATE (widget),
@@ -205,17 +204,16 @@ draw_expose_event (GtkWidget *widget,
 					x, y,
 					pl_evt);
 		      
-		      pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
+		      pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
 		      if (height < pr.height)
-                height = pr.height;
+			height = pr.height;
 		      ev->mark = TRUE;
-		      g_free (buffer);
 		    }
 		}
 	      
 	      pango_layout_set_width (pl_evt, available_width * PANGO_SCALE);
 	      pango_layout_set_text (pl_evt, evd->summary, -1);
-	      pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
+	      pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
 	      gtk_paint_layout (widget->style,
 				widget->window,
 				GTK_WIDGET_STATE (widget),
@@ -305,7 +303,6 @@ week_view_update (void)
 
   for (day = 0; day < 7; day++)
     {
-      char buf[32];
       struct week_day *d = &week_days[day];
       struct render_ctl *c = &d->rc;
 
@@ -322,10 +319,10 @@ week_view_update (void)
 		       && tm.tm_mon == today.tm_mon 
 		       && tm.tm_year == today.tm_year);
       week_days[day].is_active = week_days[day].is_today;
-      strftime (buf, sizeof (buf), "<b>%a %d %B</b>", &tm);
+
       if (d->string)
         g_free (d->string);
-      d->string = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+      d->string = strftime_strdup_utf8_utf8 ("<b>%a %d %B</b>", &tm);
 
       c->popup.day = tm.tm_mday;
       c->popup.year = tm.tm_year;
@@ -350,28 +347,21 @@ week_view_update (void)
           /* calculate width required to display times */
           PangoRectangle pr;
           event_t ev = (event_t)iter->data;
-          char buf[32];
-          char *p = buf;
-          size_t l = sizeof (buf) - 1;
-          gchar *s;
+
           if ((ev->flags & FLAG_UNTIMED) == 0)
             {
               if (ev->mark == FALSE)
                 {
-                  size_t n;
+		  gchar *s;
                   localtime_r (&ev->start, &tm);
-                  n = strftime (p, l, "%H:%M", &tm);
-                  p += n;
-                  l -= n;
-                }
-    
-              ev->mark = TRUE;
-              s = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-              pango_layout_set_text (pl_evt, s, strlen (s));
-              pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
-              if (time_width < pr.width)
-                time_width = pr.width;
-              g_free (s);
+		  s = strftime_strdup_utf8_locale ("%H:%M", &tm);
+                  ev->mark = TRUE;
+		  pango_layout_set_text (pl_evt, s, -1);
+		  pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
+		  if (time_width < pr.width)
+		    time_width = pr.width;
+		  g_free (s);
+		}
             }
         }
     }
@@ -387,19 +377,18 @@ week_view_update (void)
 
       pango_layout_set_width (pl, week_view_draw->allocation.width * PANGO_SCALE);
       pango_layout_set_markup (pl, week_days[day].string, strlen (week_days[day].string));
-      pango_layout_get_pixel_extents (pl, &pr, NULL);
+      pango_layout_get_pixel_extents (pl, NULL, &pr);
       height = pr.height + 2;
 
       if (week_days[day].events)
         {
-    
           for (iter = week_days[day].events; iter; iter = iter->next)
             {
               event_t ev = iter->data;
               event_details_t evd = event_db_get_details (ev);
               pango_layout_set_width (pl_evt, available_width * PANGO_SCALE);
               pango_layout_set_text (pl_evt, evd->summary, -1);
-              pango_layout_get_pixel_extents (pl_evt, &pr, NULL);
+              pango_layout_get_pixel_extents (pl_evt, NULL, &pr);
               height += pr.height + 2;
             }
         }
@@ -585,7 +574,7 @@ week_view_button_press (GtkWidget *widget, GdkEventButton *event, gpointer d)
 	    {
 	      int i;
 	      for (i = 0; i < 7; i++)
-            week_days[i].is_active = FALSE;
+		week_days[i].is_active = FALSE;
 	      week_days[get_day_from_y (y)].is_active = TRUE;
 	      viewtime = time_from_day (c->popup.year, c->popup.month, c->popup.day);
 	      gtk_calendar_select_month (GTK_CALENDAR (calendar), c->popup.month, c->popup.year + 1900);
@@ -595,7 +584,7 @@ week_view_button_press (GtkWidget *widget, GdkEventButton *event, gpointer d)
 	  else if (event->button == 3)
 	    {
 	      if (pop_window) 
-            gtk_widget_destroy (pop_window);
+		gtk_widget_destroy (pop_window);
     
 	      if (c != c_old) 
 		{
